Package: vitals
Title: Large Language Model Evaluation
Version: 0.3.0
Authors@R: c(
    person("Simon", "Couch", , "simon.couch@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-5676-5107")),
    person("Max", "Kuhn", , "max@posit.co", role = "ctb"),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "ctb",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Mine", "Cetinkaya-Rundel", , "mine@posit.co", role = "ctb",
           comment = c(ORCID = "0000-0001-6452-2420")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: A port of 'Inspect', a widely adopted 'Python' framework for
    large language model evaluation. Specifically aimed at 'ellmer' users
    who want to measure the effectiveness of their large language model-based
    products, the package supports prompt engineering, tool usage,
    multi-turn dialog, and model graded evaluations.
License: MIT + file LICENSE
URL: https://github.com/tidyverse/vitals, https://vitals.tidyverse.org
BugReports: https://github.com/tidyverse/vitals/issues
Depends: R (>= 4.1)
Imports: cli, dplyr, ellmer (>= 0.4.0), glue, httpuv, httr2, jsonlite,
        purrr, R6, rlang, S7, tibble, tidyr, withr
Suggests: ggplot2, here, htmltools, knitr, magick, ordinal, rmarkdown,
        testthat (>= 3.0.0), vcr (>= 2.0.0)
Config/Needs/website: tidyverse/tidytemplate, rmarkdown, btw,
        tidyverse, gt, brms, RcppEigen, broom, gt
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/usethis/last-upkeep: 2025-04-25
Config/testthat/start-first: task, translate
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-15 15:30:21 UTC; simoncouch
Author: Simon Couch [aut, cre] (ORCID: <https://orcid.org/0000-0001-5676-5107>),
  Max Kuhn [ctb],
  Hadley Wickham [ctb] (ORCID: <https://orcid.org/0000-0003-4757-117X>),
  Mine Cetinkaya-Rundel [ctb] (ORCID:
    <https://orcid.org/0000-0001-6452-2420>),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Simon Couch <simon.couch@posit.co>
Repository: CRAN
Date/Publication: 2026-05-15 20:20:02 UTC
Built: R 4.5.2; ; 2026-05-15 22:08:32 UTC; unix
