# Generated by roxygen2: do not edit by hand

S3method(print,Task)
export(Task)
export(detect_answer)
export(detect_exact)
export(detect_includes)
export(detect_match)
export(detect_pattern)
export(generate)
export(generate_structured)
export(model_graded_fact)
export(model_graded_qa)
export(vitals_bind)
export(vitals_bundle)
export(vitals_log_dir)
export(vitals_log_dir_set)
export(vitals_view)
if (getRversion() < "4.3.0") importFrom("S7", "@")
import(rlang)
importFrom(R6,R6Class)
importFrom(cli,cat_line)
importFrom(cli,col_grey)
importFrom(cli,format_inline)
importFrom(ellmer,content_image_url)
importFrom(tibble,tibble)
importFrom(withr,defer)
