#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import rlang
#
# quiet "All declared Imports should be used" NOTE
#' @importFrom R6 R6Class
#' @importFrom ellmer content_image_url
#' @importFrom tibble tibble
#' @importFrom withr defer
## usethis namespace: end
NULL

utils::globalVariables(c(
  "any_of",
  "capture.output",
  "chat_claude",
  "everything",
  "head",
  "input",
  "input_before",
  "model",
  "output",
  "output_before",
  "path",
  "price",
  "price_before",
  "provider",
  "qnorm",
  "res",
  "setNames",
  "str",
  "timestamp",
  "var"
))
