# generate() allows NULL default model

    Code
      res()
    Condition
      Error in `res()`:
      ! `solver_chat` must be a <Chat>, not NULL

# generate_structured() allows NULL default model

    Code
      res()
    Condition
      Error in `res()`:
      ! `solver_chat` must be a <Chat>, not NULL

